//
// Copyright 2013, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
#include <cxxtls/CursorWindowViewportAdjust.h>

namespace cxxtls
{

void resizeAllViewports(WindowAdjustments::ResizeMap & currentSizes,
                        int newTotalWidth
                       )
    ///  Given a sorted collection of window viewport spans, handle an overall change
    ///  to the size.

{
    const int wScale = 100000;

    WindowAdjustments::ResizeMap::iterator last = currentSizes.end(),
                                           first,
                                           next;

    int oldTotalWidth=0;

    if(last == currentSizes.begin())
       return;

    --last;

    oldTotalWidth = last->second.originalSpan_.origin_ + last->second.originalSpan_.width_;

    if(oldTotalWidth == 0)
      oldTotalWidth = 1;

    last = currentSizes.end();

    // scale the origin of each viewport to the new size 

    for(first = currentSizes.begin(); first != last; ++first)
    {
        WindowAdjustments::Span *curSpan = &first->second.currentSpan_;

        curSpan->origin_ *= newTotalWidth * wScale;
        curSpan->origin_ /= wScale;
        curSpan->origin_ /= oldTotalWidth;
    }

    // add an extra element to the map to make the algorith easier.
    // we are going to iterate from first to last, but need to be able to
    // safely access last+1 with correct data in it.

    currentSizes.insert(WindowAdjustments::ResizeMap::value_type(oldTotalWidth, 
                                                                 WindowAdjustments::ResizeInfo(&last, 
                                                                                               newTotalWidth, 
                                                                                               0, 
                                                                                               oldTotalWidth, 
                                                                                               0)));

    last = currentSizes.end();

    --last;  // range of interest is 1 less than total size of currentSizes, but we are
             // going to access last+1 in the algorithm

    // now compute the new width of each span

    for(first = currentSizes.begin(); first != last; ++first)
    {
        WindowAdjustments::ResizeMap::value_type *cur = &*first;
        next = first;
        ++next;

        WindowAdjustments::ResizeMap::value_type *nxt = &*next;

        cur = cur; // keep for the debugger
        nxt = nxt;

        // despite the appearance, the "next" item may not actually be the
        // real next item.  Here's a picture of why
        //
        // +--------------------+
        // |        a           |
        // +----------+---------+
        // |    b     |   c     |
        // +----------+---------+
        //
        // "a" and "b" could appear in the map in either order, but
        // "c" is not the "next" for "a", only for b".  So we must
        // scan to find the real next

        int oldOrigin = first->second.originalSpan_.origin_;
        int oldWidth  = first->second.originalSpan_.width_;

        int oldNextEnd = oldOrigin + oldWidth;
        
        while(next != currentSizes.end() && next->second.originalSpan_.origin_ < oldNextEnd)
        {
           ++next;
           nxt = &*next;

        }

        if(next == currentSizes.end())
        {
           next = first;
           ++next;
        }

        int nextOrigin = next->second.currentSpan_.origin_;
        int curOrigin  = first->second.currentSpan_.origin_;
        int width      = nextOrigin - curOrigin;

        first->second.currentSpan_.width_ = width;

    }


    
}

}
