//
// Copyright 2012, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
#ifdef __linux__
#include <cxxtls/terminalstatus.h>

#include <term.h>
#include <termios.h>


namespace cxxtls
{
    struct TerminalState
    :  public termios
    {
    };

    TerminalStatus::
    TerminalStatus(int fd)
    :  override_termios_(new TerminalState)
    {
        if(0 > tcgetattr(fd, override_termios_.get()))   // record stdin's term characteristics 
           failed_ = true;
        else
           failed_ = false;
    }

    TerminalStatus::
    TerminalStatus(TerminalStatus const &rhs)
    :  override_termios_(new TerminalState)
    {
        failed_ = rhs.failed_;

        *override_termios_ = *rhs.override_termios_;
    }

    TerminalStatus&
    TerminalStatus::
    operator=(TerminalStatus const &rhs)
    {
        failed_ = rhs.failed_;

        *override_termios_ = *rhs.override_termios_;

        return *this;
    }


    void 
    TerminalStatus::
    setRaw()
    {
       if(!failed_)
       {
           cfmakeraw(override_termios_.get());
       }
    }

    void
    TerminalStatus::
    setNoEcho()
    {
        override_termios_->c_iflag  &= (~ECHO);   
    }

    void
    TerminalStatus::
    setSignal(bool enabled)
    {
        if(!enabled)
        {
            override_termios_->c_lflag  &= (~ISIG);   
        }
        else
        {
            override_termios_->c_lflag  |= ISIG;   
            override_termios_->c_iflag  |= BRKINT;
        }
    }


    int 
    TerminalStatus::
    setStatus(int fd)
       /// force the terminal status on a file handle.  @return a negative number
       /// if the operation fails.  See errno
    {
        if(!failed_)
            return tcsetattr(fd, TCSANOW, override_termios_.get());

        return -1;
    }


}
#else
   int please_ignore_terminalstatus_cxx_on_windows=0;
#endif
